#include "filestreemodel.h"

FilesTreeItem::FilesTreeItem(const QList<QVariant> &data, FilesTreeItem *parent)
{
    parentItem = parent;
    itemData = data;
}

FilesTreeItem::~FilesTreeItem()
{
    qDeleteAll(childItems);
}

void FilesTreeItem::appendChild(FilesTreeItem *item)
{
    childItems.append(item);
}

FilesTreeItem * FilesTreeItem::child(int row)
{
    return childItems.value(row);
}

int FilesTreeItem::childCount() const
{
    return childItems.count();
}

int FilesTreeItem::columnCount() const
{
    return itemData.count();
}

QVariant FilesTreeItem::data(int column) const
{
    return itemData.value(column);
}

FilesTreeItem * FilesTreeItem::parent()
{
    return parentItem;
}

int FilesTreeItem::row() const
{
    if (parentItem)
        return parentItem->childItems.indexOf(const_cast<FilesTreeItem*>(this));

    return 0;
}

FilesTreeModel::FilesTreeModel(libtorrent::file_storage const& files, QObject * parent)
    : QAbstractItemModel(parent)
{
    QList<QVariant> rootData;
    rootData << "File" << "Download" << "Priority";
    rootItem = new FilesTreeItem(rootData);
    setupModelData(files, rootItem);
}

FilesTreeModel::~FilesTreeModel()
{
    delete rootItem;
}

int FilesTreeModel::columnCount(const QModelIndex &parent) const
{
    if (parent.isValid())
        return static_cast<FilesTreeItem*>(parent.internalPointer())->columnCount();
    else
        return rootItem->columnCount();
}

QVariant FilesTreeModel::data(const QModelIndex &index, int role) const
{
    if (!index.isValid())
        return QVariant();

    if (role != Qt::DisplayRole)
        return QVariant();

    FilesTreeItem *item = static_cast<FilesTreeItem*>(index.internalPointer());

    return item->data(index.column());
}

Qt::ItemFlags FilesTreeModel::flags(const QModelIndex &index) const
{
    if (!index.isValid())
        return 0;

    return Qt::ItemIsEnabled | Qt::ItemIsSelectable;
}

QVariant FilesTreeModel::headerData(int section, Qt::Orientation orientation,
                               int role) const
{
    if (orientation == Qt::Horizontal && role == Qt::DisplayRole)
        return rootItem->data(section);

    return QVariant();
}

QModelIndex FilesTreeModel::index(int row, int column, const QModelIndex &parent)
            const
{
    if (!hasIndex(row, column, parent))
        return QModelIndex();

    FilesTreeItem *parentItem;

    if (!parent.isValid())
        parentItem = rootItem;
    else
        parentItem = static_cast<FilesTreeItem*>(parent.internalPointer());

    FilesTreeItem *childItem = parentItem->child(row);
    if (childItem)
        return createIndex(row, column, childItem);
    else
        return QModelIndex();
}

QModelIndex FilesTreeModel::parent(const QModelIndex &index) const
{
    if (!index.isValid())
        return QModelIndex();

    FilesTreeItem *childItem = static_cast<FilesTreeItem*>(index.internalPointer());
    FilesTreeItem *parentItem = childItem->parent();

    if (parentItem == rootItem)
        return QModelIndex();

    return createIndex(parentItem->row(), 0, parentItem);
}

int FilesTreeModel::rowCount(const QModelIndex &parent) const
{
    FilesTreeItem *parentItem;
    if (parent.column() > 0)
        return 0;

    if (!parent.isValid())
        parentItem = rootItem;
    else
        parentItem = static_cast<FilesTreeItem*>(parent.internalPointer());

    return parentItem->childCount();
}

void FilesTreeModel::setupModelData(libtorrent::file_storage const& files, FilesTreeItem *parent)
{

    for (libtorrent::file_storage::iterator itr = files.begin(); itr != files.end(); itr++)
    {
        QList<QVariant> l;
        l << "hi" << "hi2" << "hi3";
        parent->appendChild(new FilesTreeItem(l, parent));
    }


    /*QList<FilesTreeItem*> parents;
    QList<int> indentations;
    parents << parent;
    indentations << 0;

    int number = 0;

    while (number < lines.count()) {
        int position = 0;
        while (position < lines[number].length()) {
            if (lines[number].mid(position, 1) != " ")
                break;
            position++;
        }

        QString lineData = lines[number].mid(position).trimmed();

        if (!lineData.isEmpty()) {
            // Read the column data from the rest of the line.
            QStringList columnStrings = lineData.split("\t", QString::SkipEmptyParts);
            QList<QVariant> columnData;
            for (int column = 0; column < columnStrings.count(); ++column)
                columnData << columnStrings[column];

            if (position > indentations.last()) {
                // The last child of the current parent is now the new parent
                // unless the current parent has no children.

                if (parents.last()->childCount() > 0) {
                    parents << parents.last()->child(parents.last()->childCount()-1);
                    indentations << position;
                }
            } else {
                while (position < indentations.last() && parents.count() > 0) {
                    parents.pop_back();
                    indentations.pop_back();
                }
            }

            // Append a new item to the current parent's list of children.
            parents.last()->appendChild(new FilesTreeItem(columnData, parents.last()));
        }

        number++;
    }*/
}
