#ifndef FILESTREEMODEL_H
#define FILESTREEMODEL_H

#include <QList>
#include <QVariant>
#include <QAbstractItemModel>
#include <QModelIndex>
#include <QStringList>

#include "libtorrent/storage.hpp"

class FilesTreeItem
{
public:
    FilesTreeItem(const QList<QVariant> &data, FilesTreeItem *parent = 0);
    ~FilesTreeItem();

    void appendChild(FilesTreeItem *child);

    FilesTreeItem * child(int row);
    int childCount() const;
    int columnCount() const;
    QVariant data(int column) const;
    int row() const;
    FilesTreeItem *parent();

private:
    QList<FilesTreeItem*> childItems;
    QList<QVariant> itemData;
    FilesTreeItem *parentItem;
};

class FilesTreeModel : public QAbstractItemModel
{
    Q_OBJECT
public:
    FilesTreeModel(libtorrent::file_storage const& files, QObject *parent = 0);
    ~FilesTreeModel();

    QVariant data(const QModelIndex &index, int role) const;
    Qt::ItemFlags flags(const QModelIndex &index) const;
    QVariant headerData(int section, Qt::Orientation orientation,
                        int role = Qt::DisplayRole) const;
    QModelIndex index(int row, int column,
                      const QModelIndex &parent = QModelIndex()) const;
    QModelIndex parent(const QModelIndex &index) const;
    int rowCount(const QModelIndex &parent = QModelIndex()) const;
    int columnCount(const QModelIndex &parent = QModelIndex()) const;

private:
    void setupModelData(libtorrent::file_storage const& files, FilesTreeItem *parent);

    FilesTreeItem *rootItem;
};

#endif // FILESTREEMODEL_H
